import discord, traceback
from discord.ext import commands

class EventCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    # Sets the game and logs when the bot logs in
    @commands.Cog.listener()
    async def on_ready(self):
        self.bot.log("Logged in as "+self.bot.user.name+", "+str(self.bot.user.id), level = 20)
        game = discord.Game("Use /help for commands")
        await self.bot.change_presence(activity=game)

    # Logs when user sends a command
    @commands.Cog.listener()
    async def on_slash_command(self, ctx):
        self.bot.log(f"{ctx.author.display_name} ran /{ctx.name}", str(ctx.channel_id), level = 20)

    # Logs if a command experiences an error
    @commands.Cog.listener()
    async def on_slash_command_error(self, ctx, error):
        if isinstance(error, commands.CommandNotFound):
            await ctx.send("That's not a command (error code 001)")
        elif isinstance(error,commands.errors.MissingRequiredArgument):
            self.bot.log(f"{error}",str(ctx.channel_id))
            await ctx.send("Missing command parameters (error code 002). Try using `!help [command]` to find out how to use the command.")
        else:
            exception = traceback.format_exception(type(error), error, error.__traceback__)
            stopAt = "\nThe above exception was the direct cause of the following exception:\n\n"
            if stopAt in exception:
                index = exception.index(stopAt)
                exception = exception[:index]

            exceptionString = "".join(exception)
            self.bot.log([f"exception in /{ctx.name}", f"{exceptionString}"],str(ctx.channel_id), 40)
            await ctx.send("Something went wrong (error code 000)")

def setup(bot):
    bot.add_cog(EventCog(bot))
